/*
	Provides a SmartBar interface for websites and Apps.
	
	Future Goals:
	1. Fully compatible with non webkit browsers.
	2. Fully frothjs compatible.

	Documentation
	
	Bar div wrapper class 
	.smart-bar
	
	Left Items Wrapper (overflow)
	.smart-bar .left-items
 
 	overflowable (.overflow)
 	auto hide the overflow-menu item when all items display (.auto-hide)
 
 	Overflow items get wrapped into a hidden select with the options behaving just like their hidden items
 
	Right Items Wrapper (static, absolute)
	.smart-bar .right-items
 
	Item
	span.sbitem
 
	Selected Item
	span.sbitem.selected
 
	Overflow Item
	span.sbitem.overflow-menu
	
	Triggers a popup
	span.sbitem.popup
*/

var MoreOverFlowWidth = 50;

function UISmartItemWithIdentifier(id) {
	return $('div.tasks-smart-bar span#'+id)
}

function UISelectSmartItemWithIdentifier(id) {
	UISmartItemWithIdentifier(id).siblings('.sbitem').removeClass('selected');
	UISmartItemWithIdentifier(id).addClass('selected');
}

function UIRemoveAllLeftSmartItems(barid) {
	$('#'+barid+'.smart-bar > div.left-items span').remove();
	//UIAdjustOverflowSizingForBar('#'+barid+'.smart-bar');
}

function UIGetLeftSmartItemsContainer(barid) {
	return $('#'+barid+'.smart-bar > div.left-items');
}

function UIAdjustOverflowSizingForBar(bar) {
	bar.each(function() {
		var leftdiv = $(this).find('.left-items');
	
		var full = $(this).width();
		var left = leftdiv.outerWidth() + MoreOverFlowWidth;
		var right = $(this).find('.right-items').outerWidth();
		
		//constraint left to full - right
		var c = full - right;
		
		
		//NSLog('bar width:'+ full + ' left: '+ left + ' right:' + right + ' constraint: ' + c);
		
		//Calculate the over width and remove the applicable amount of items to overflow
		//NSLog('cutoff: ' + cuttoff);
		
		var hasOverflow = false;
		var length = MoreOverFlowWidth;
		leftdiv.find('.sbitem:not(.overflow-menu)').each(function() {
			length = length + $(this).outerWidth() + 4; //margin
			if(length > c) {
				$(this).addClass('overflowed');
				hasOverflow = true;
			} else {
				$(this).removeClass('overflowed');
			}
		});
			
		
		if(hasOverflow) {
			if(leftdiv.find('span.overflow-menu').length < 1) {
				leftdiv.append("<span class='sbitem overflow-menu'>▾ More</span>");
			}
			//Process select group
			processSelectForOverflowMenu(leftdiv);
		} else {
			leftdiv.find('.sbitem.overflow-menu').remove();
		}
	});
}

/*
 Private Initalizations and bindings
*/

function processSelectForOverflowMenu(sidebar) {
	//get all sbitem.overflowed
	var overflowmenu = sidebar.find('.sbitem.overflow-menu');
	overflowmenu.find('select').remove();
	overflowmenu.append('<select/>');
	
	
	overflowmenu.find('select').change(function(){
		var selected = $(this).find('option:selected').attr('value');
		sidebar.find('#'+selected+'.sbitem').click();
		sidebar.find('.sbitem.overflow-menu').addClass('selected');
	});
	
	
	overflowmenu.removeClass('selected');
	sidebar.find('.sbitem.overflowed').each(function() {
		if($(this).hasClass('selected')) {
			overflowmenu.find('select').append("<option value='"+$(this).attr('id')+"' selected='selected'>"+$(this).text()+"</option>");
			overflowmenu.addClass('selected');
		} else {
			overflowmenu.find('select').append("<option value='"+$(this).attr('id')+"'>"+$(this).text()+"</option>");
		}
	});	
}

$(function() {
	$(".smart-bar span.sbitem:not(.popup):not(.toggle):not(.overflow-menu)").die('click').live('click', function(e) {
		if(!$(this).hasClass('selected')) {
			$(this).siblings('.sbitem').removeClass('selected');
			$(this).addClass('selected');
			
			if(!$(this).hasClass('overflowed')) {
				$(this).closest('.left-items').find('select > option').removeAttr('selected');
				$(this).closest('.left-items').find('.sbitem.overflow-menu').removeClass('selected');
			} else {
				$(this).closest('.left-items').find('.sbitem.overflow-menu').addClass('selected');
			}
			
			//Remove this... instead fire a event 'sbselect' custom event on the smartbar with the id as a value
			SmartGroupBarController.smartBarItemSelected_($(this).attr('id'));	
		}
	});
	
	//Set initial sizes (should also be called when items are added or removed.
	UIAdjustOverflowSizingForBar($('.smart-bar'));
	
	/*left-items .overflow .auto-hide support*/
	if($('.smart-bar .left-items.overflow').length > 0) {
		$(window).resize(function(e) {
			UIAdjustOverflowSizingForBar($('.smart-bar'));
		});
	}
});